function [Caponc1,wCaponc1] = Capon_impl8(xx,Zetat,LL,Nsnap,spann)
%   Capon_impl8.m
%   Ole Faltin, electronics student at NTNU, as part of the 
%   project Estimation of harmonic signatures in underwater noise
%   from small boats
%   Masters Thesis for FFI - Dag Tollefsen
%   06.06.2015
%
%   Method proposed by 
%   Christensen, M. G. and Stoica, P. and Jensen, S. H.
%   The Multi-Pitch Estimation Problem: Some New Solutions
%   IEEE International Conference on Acoustics, Speech, and  
%   Signal Processing, 3(1), 1221-1224.
%
%   Capon estimator implementation
%   Calulates the fundamental frequency and the cost function. 
%   
%   In:
%   xx    - Signal vector.
%   Zetat - Array of angular frequencies where the fundamental is
%   being searched for.
%   LL    - Number of harmonics in the signal.
%   Nsnap - Snapshot length
%   spann - regression span - 0<spann<=1
%
%   Out:
%   Caponprod1  - Cost function calculated using Capon.
%   wCaponprod1 - Weighted cost function
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Common parameters                 
Caponc1 = zeros(length(Zetat),1);    
sTYPE = 'lowess';

RR = acorrmat(xx);          % Requires acorrmat.m

[EVEC,EVAL] = eig(RR);      % Eigen decomposition; assume real xx
EPROD = EVEC * diag(1./sqrt(diag(EVAL)));

kk = 0;
while kk < length(Zetat)    % Will execute once for each angular f.
                            % w in Zetat
    kk = kk+1;
    zetakk = Zetat(kk);          % extract current zeta
    ll = 1:LL(kk);
    
    % Make signal model for current w
    ZZ = exp(1j*zetakk*(0:(Nsnap-1)).'*(ll));
    
    % Calcualte product and take trace
    Y3 = pinv(ZZ' * EPROD * (ZZ' * EPROD)'); 
    Caponc1(kk) = abs(trace(Y3));
end

% Perform smooting
yy = smooth((Caponc1),spann,sTYPE);
wCaponc1 = (Caponc1) - (yy);

end