function [NLSprod1,wNLSprod1] = NLS_impl8(xx,Zetat,LL,Nsnap,spann) 
%   NLS_impl8.m
%   Ole Faltin, electronics student at NTNU, as part of the 
%   projectEstimation of harmonic signatures in underwater noise  
%   from small boats
%   Masters Thesis for FFI - Dag Tollefsen
%   06.06.2016
%
%   Method proposed by 
%   Christensen, M. G. and Stoica, P. and Jensen, S. H.
%   The Multi-Pitch Estimation Problem: Some New Solutions
%   IEEE International Conference on Acoustics, Speech, and Signal 
%   Processing, 3(1), 1221-1224.
%
%   NLS Estimator implementation
%   Calulates the fundamental frequency and the cost function. 
%   
%   In:
%   xx    - Signal vector.
%   Zetat - Array of angular frequencies where the fundamental is
%   being searched for.
%   LL    - Number of harmonics in the signal.
%   Nsnap - Snapshot length
%   spann - regression span - 0<spann<=1
%
%   Out:
%   NLSprod1    - Cost function calculated using NLS.
%   wNLSprod1   - Weighted cost function
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NLSprod1 = zeros(length(Zetat),1);
sTYPE = 'lowess';           % Type of smoothing. default: LOWESS

kk = 0;
while kk < length(Zetat)    % Will execute once for each angular f.
    kk = kk+1;              
    zetakk = Zetat(kk);     % extract current zeta
    
    ll = 1:LL(kk);          % Make ll for current w - LL(kk)
    
    % Make Vandermonde matrix
    ZZ = exp(1j*zetakk*(0:(Nsnap-1)).'*(ll));

    NLSprod1(kk) = xx' * ZZ  * ZZ' * xx;
end
NLSprod1 = abs(NLSprod1);
% Perform smooting
yy = smooth(NLSprod1,spann,sTYPE);
wNLSprod1 = (NLSprod1) - (yy);

end