function [RR] = acorrmat(xx)
%   acorrmat.m
%   Ole Faltin, electronics student at NTNU, as part of the project
%   Estimation of harmonic signatures in underwater noise from 
%   small boats
%   Masters Thesis for FFI - Dag Tollefsen
%   11.05.2015
%
%   Autocorrelation matrix implementation
%   Calcualtes the autocorrelation matrix of a signal snapshot. 
%   
%   In:
%   xx - Signal snapshot vector. Should be real.
%
%   Out:
%   RR - Autocorrelation matrix.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%   Common parameters
Nsnap = length(xx);         % Length of snapshot
mx = mean(xx);              % Mean value of snapshot
rr = zeros(1,Nsnap);        % Row vector for top row results

%   Calculation
for k = 0:(Nsnap-1)
    r = 0;
    for n = 1:Nsnap-k       % Correlation
        r = r + sum((xx(n)-mx)*(xx(n+k)-mx)');
    end
    rr(1,k+1) = r;
end

%   Generate first collumn.
cc=[rr(1),conj(rr(2:Nsnap))].';

%   Use toeplitz.m to generate proper RR = autocorrelation matrix.
RR = toeplitz(cc,rr);

end