%   performer.m
%   Ole Gunnar Mathisen Faltin, electronics student at NTNU, as part of 
%   the project "Estimation of harmonic signatures in underwater noise 
%   from small boats"
%   for FFI - Dag Tollefsen
%
%   Supervisor at NTNU - Hefeng Dong
%   
%   Methods proposed by 
%   Christensen, M. G. and Stoica, P. and Jensen, S. H.
%   The Multi-Pitch Estimation Problem: Some New Solutions
%   IEEE International Conference on Acoustics, Speech, and Signal 
%   Processing, 3(1), 1221-1224.
%
%   The files NLS_impl8.m and Capon_impl8.m with acorrmat.m are needed
%   in order to execute the file.
%
%   Spectrogram with lines might have a problem in MATLAB2015+
%
%   Do not hesitate to contant me at ole.faltin@gmail.com for information
%
%   Ole Faltin
%   06.06.2015
%
%% Script control

syntheticcheck = 1;         % 1 to generate noise, else calcualte with 
                            % signal form file.
                            
%   Synthetic only
excludecheck = 0;           % 1 to use excluded harmonics 
excludedHarmonics = [1:3:50];% vector of excluded harmonics
dampconst = 2;              % Amount of dampening on synth. harm.
sigmaval = 2;               % STD of synthetic noise
SigLenSec = 4;              % Signal length in seconds for synth sig

SnapLenFrac = 1/6;          % how long snapshots compared to fs?
pss = 1;                    % Snapshot to study in detail
OR = 0.0;                   % overlap ratio - calculate with overlapping
                            % snapshots
spann = 0.15;               % Local smoothing span
sTYPE = 'lowess';           % Type of smoothing. LOWESS is used by default
                            % in implementation files.
                            % See smooth.m reference pages.

%% Get signal
% if syntheticcheck = 1: synthetic signal, else get from file
                            
if syntheticcheck ~= 1;
    % For older versions of MATLAB wavread is probably usefull
    %[totsig,fssp] = audioread('Data/file'); 
    [totsig,fssp] = audioread('path/filename.wav');
    % Files used in thesis:
    % acu140624-131549_ch00_143410.wav          Boat1
    % acu140624-131549_ch00_151206.wav          Boat2
    % acu140624-111419_ch00_123119_ny.wav       Boat3
    % acu140625-012453_ch00_024453_stoey1.wav   Ambient
    
    DecFact = 3;
    signal = decimate(totsig,DecFact,'fir');
    fs = fssp/DecFact;
                
elseif syntheticcheck == 1; 
    disp('Making synthetic signal');

    if(SigLenSec)>10
        disp('Warning: Calcualting with a long signal.') 
        disp('If you are on a laptop, it is recommended to edit') 
        disp('the variable: SigLenSec')
    end
                                    
    LLsig = 55;                     % Number of harmonics
    fs = 3000;                      % Sample frequency
    f_fund = 26.3690643;            % Fundamental frquency
    
    
    TotLend = fs*SigLenSec;         % Signal length in samples
    tt = 1:TotLend;

    fampl = real(exp(-dampconst*(0:(LLsig-1))/LLsig));

    harmonics = (1:LLsig);         % Vector of harmonics
    ff = harmonics*f_fund;      % Vector of harmonic frequencies
    phi_n = 2*pi*(rand(1,LLsig));  % Phase of signals

                                % the fundamental.
    signals = 1;                         

    % Generate signal - Old code - signal_noise_generator.m

    signal = zeros(1,TotLend);   % Signal vector
    signale = zeros(1,TotLend);  % Excluded harmonics signal vector

    %   Harmonic frequency amplitudes.
    %   Take into account excluded harmonics
    if excludecheck == 1
        %fampl = fampl;
        harmonicse = harmonics;
        for har = excludedHarmonics
            pos = find(harmonicse == har);   
            harmonicse(pos) = 0;
            fampl(pos) = 0;
        end

        ffe = harmonicse*f_fund;    % vector of excluded hamronic freqs
    end


    % Generate harmonics and put them in signal
    for ii = 1:LLsig
        f = ff(ii);
        for ttt = 1:TotLend
            t = ttt/fs;
            csig = exp(1j*2*pi*f*t+1j*phi_n(ii));
            signal(ttt) = signal(ttt) + fampl(ii)* csig;
            %if excludecheck == 1    % Excluded harmonics
            %    signale(ttt) = signale(ttt) + fample(ii)*csig;
            %end
        end   
    end

    % Geenrate noise and add it to signal
    for snrit = 1:length(sigmaval) 
        % snrit to continue old simulations

        sigma = sigmaval(snrit);

        %increase accuracy of SNR
        nn1 = (sigma)*randn(size(signal)); %k*randn; var(nn) = k^2
        nrms = sqrt(mean(abs(nn1).^2));

        srms = sqrt(mean(abs(signal).^2));

        SNR = 20*log10(srms/nrms);
            
        signal = signal + nn1;
    end

    fprintf('SNR of synthetic signal is %g \n',SNR)
end

         

%%  Initiation

%   Script control
SnapLenNLS = round(SnapLenFrac*fs); % snapshot length in samples NLS
SnapLenCapon = SnapLenNLS;          

fprintf('Snapshot length NLS: %g \n',SnapLenNLS)
fprintf('Snapshot length Capon: %g \n',SnapLenCapon)

Dt = 1/fs;                  % Time increment


LenNLS = SnapLenNLS/fs;           % Signal length in seconds
LenCapon = SnapLenCapon/fs;
TotLent = length(signal)/fs;    % Total signal length in seconds
TotLend = length(signal);
f_tak = fs/2;


fftlenNLS = 2^ceil(log2(SnapLenNLS)+4);
fftlenCapon = 2^ceil(log2(SnapLenCapon)+4);


fminn = 10; %5? %10
fmaxx = 65;%100;            % Maximum test frequency
FreqSize = round(5001*(fmaxx-fminn)/100);  % Test frequecies
progress_long = 0;          % Progress parameter - used to display progress

fftt = linspace(fminn,fmaxx,FreqSize);
LL = fftt;

koeff = 2*pi/fs;            % Convertion factor
fftlength = 2^(ceil(log2(SnapLenNLS))+10);

f_d = fmaxx/(FreqSize-1);
Zetat = fftt * koeff;       % Range of frequencies where we expect to find
                            % the fundamental.


 

% Calcualte number of harmonics
maxharm = 100;              % maximum number of harmonics

for kk = 1:length(LL)
    LL(kk) = floor(f_tak/LL(kk));
    if LL(kk) > maxharm
        LL(kk) = maxharm;
    end

end

figure(50)
plot(fftt,LL)
title('Number of possible harmonics for a frequencies in \zeta')
xlabel('Fundamental frequency, \zeta [Hz]'), ylabel('Harmonics')
ylim([0,maxharm+10])
grid on;
xlim([fminn, fmaxx]);


% Calculating different parameters:

                                                
                            
%%  Get signal

SnapShotNumbNLS = floor((TotLend/SnapLenNLS)/(1-OR));%Finding number of snapshots
SnapShotNumbCapon = floor((TotLend/SnapLenCapon)/(1-OR));%Finding number of snapshots

%   Debug for real data: calculate with fewer snapshots
%SnapShotNumbNLS = 150;
%SnapShotNumbCapon = 150;


%   Time indexing for each snapshot
Tsnap0NLS = (0:(SnapShotNumbNLS-1))*SnapLenNLS + 1;
Tsnap0Capon = (0:(SnapShotNumbCapon-1))*SnapLenCapon + 1;

if OR ~= 0
    Tsnap0NLS = (0:(SnapShotNumbNLS-1))*SnapLenNLS*OR + 1;
    Tsnap0Capon = (0:(SnapShotNumbCapon-1))*SnapLenCapon*OR + 1;
end

%   Matrix for recieved signal (noise)
ResSigNLS = zeros(SnapShotNumbNLS,SnapLenNLS);
ResSigCapon = zeros(SnapShotNumbCapon,SnapLenCapon);
ttNLS = ResSigNLS;
ttCapon = ResSigCapon;

%   Populate recieved matrix, rr, orginized as snaps.
SnapMatDimNLS = size(ResSigNLS);
SnapMatDimCapon = size(ResSigCapon);

%   NLS
for SnapNum = 1:SnapMatDimNLS(1)
    curTsnap0NLS = Tsnap0NLS(SnapNum);
    curSnap = signal(curTsnap0NLS:(curTsnap0NLS+SnapLenNLS-1));
    
    ResSigNLS(SnapNum,:) = curSnap;
    ttNLS(SnapNum,:) = curTsnap0NLS:(curTsnap0NLS+SnapLenNLS-1);
    
end

%   Capon
for SnapNum = 1:SnapMatDimCapon(1)
    curTsnap0Capon = Tsnap0Capon(SnapNum);
    curSnap = signal(curTsnap0Capon:(curTsnap0Capon+SnapLenCapon-1));
    
    ResSigCapon(SnapNum,:) = curSnap;
    ttCapon(SnapNum,:) = curTsnap0Capon:(curTsnap0Capon+SnapLenCapon-1);
    
end



%%  Defining result vectors


NLSFreqVect = zeros(SnapShotNumbNLS,1);
CaponFreqVect = zeros(SnapShotNumbCapon,1);

wNLSFreqVect = zeros(SnapShotNumbNLS,1);
wCaponFreqVect = zeros(SnapShotNumbCapon,1);

TimeNLS = zeros(SnapShotNumbNLS,1);
TimeCapon = zeros(SnapShotNumbCapon,1);
totTime = 0;

%   Vectors for normalized cost functions
nCostNLS = zeros(SnapShotNumbNLS,FreqSize);
nCostCapon = zeros(SnapShotNumbCapon,FreqSize);

FreqNLS = zeros(SnapShotNumbNLS,1);
FreqCapon = zeros(SnapShotNumbCapon,1);
CostNLS = zeros(SnapShotNumbNLS,FreqSize);
CostCapon = zeros(SnapShotNumbCapon,FreqSize);
wCostNLS = zeros(SnapShotNumbNLS,FreqSize);
wCostCapon = zeros(SnapShotNumbCapon,FreqSize);

CaponHits = zeros(SnapShotNumbCapon,1);
NLSHits = zeros(SnapShotNumbNLS,1);


%% NLS estimation
TotSnapShots = SnapShotNumbNLS + SnapShotNumbCapon;

tic
%   Can be executed as for loop (change parfor to for)
parfor sigit = 1:SnapShotNumbNLS %1 to for debugging
    xx = real(ResSigNLS(sigit,:)).';    % Take a snapshot
    
    % Perform estimation
    tic
    [CostNLS(sigit,:),wCostNLS(sigit,:)] = NLS_impl8(...
        xx,Zetat,LL,SnapLenNLS,spann);
    TimeNLS(sigit) = toc;

end

totnlstime = toc;
xx = real(ResSigNLS(1,:));
% Normalizing
Nsnap = SnapLenNLS;

for ss = 1:SnapShotNumbNLS
    nCostNLS(ss,:) = CostNLS(ss,:)/max(CostNLS(ss,:));
    pp = find(nCostNLS(ss,:) == 1);
    NLSFreqVect(ss) = fftt(pp);
    
    nwCostNLS(ss,:) = wCostNLS(ss,:)/max(wCostNLS(ss,:));
    pp = find(nwCostNLS(ss,:) == 1);
    wNLSFreqVect(ss) = fftt(pp);
    
    ll = 1:LL(pp);
    w = Zetat(pp);
    zznls = exp(1j*w*(0:(Nsnap-1)).'*(ll));

    aaahatt = pinv(zznls'*zznls)*zznls'*xx.';
    figure(3);
    hold on;
    grid on;
    plot(ll,abs(aaahatt));
end

figure(3)
title('Harmonic amplitudes for estimated signals')
xlabel('Snapshot number'), ylabel('Relative amplitude.')

fprintf('NLS estimation performed in %g seconds. \n',totnlstime)




%% Capon estiamtion
disp('Capon estimation started.')
disp('This will likely be more tedious than NLS.')


%   Can be executed as for loop (change parfor to for)
parfor sigit = 1:SnapShotNumbCapon %1 to for debugging % fra 334
    xx = real(ResSigCapon(sigit,:)).';
    
    %   Calling Capon implementation
    tic
    [CostCapon(sigit,:),wCostCapon(sigit,:)] = Capon_impl8(...
        xx,Zetat,LL,SnapLenCapon,spann);
    TimeCapon(sigit) = toc;
    
end

disp('Capon estimation performed.')


%%  Normalizing the cost functions

for ss = 1 : SnapShotNumbCapon
    nCostCapon(ss,:) = abs(CostCapon(ss,:))/max(abs(CostCapon(ss,:)));
    pp = find(nCostCapon(ss,:) == 1);
    CaponFreqVect(ss) = fftt(pp);
    
    nwCostCapon(ss,:) = wCostCapon(ss,:)/max(wCostCapon(ss,:));
    pp = find(nwCostCapon(ss,:) == 1);
    
    wCaponFreqVect(ss) = fftt(pp);
    
    
end

disp('Cost functions normalized.')

%%  Plotting
disp('Plotting a varyity of figures. Do not touch anything now!')
pause(3)
    
%   Calculate frequency content of signals

SS_0 = fft(real(ResSigNLS(1,:)),fftlength)/SnapLenNLS;
ww_0 = fs/2*linspace(0,1,fftlength/2+1);

%   Plotting frequency content
close(figure(2))
figure(2)
plot(ww_0,2*abs(SS_0(1:fftlength/2+1))); 
xlim([0, fs/2])
title('SS -  FFT of signal')% (first snapshot)')
ylabel('Normalized amplitude'), xlabel('Frequency [Hz]')
grid on;


fig1=24;


figure(fig1 );
plot(fftt, abs(CostNLS(:,:)))
title('Non-linear Least Squares cost function')
xlabel('Frequency [Hz]'), ylabel('Cost function')
grid on;

figure(fig1 + 1);
plot(fftt, abs(CostCapon(:,:)))
title('Capon cost function')
xlabel('Frequency [Hz]'), ylabel('Cost function')
grid on;


figure(fig1 + 2);
plot(fftt,abs(nCostNLS(pss,:)),fftt, abs(nCostCapon(pss,:)))
title('Cost functions for interesting snapshot (pss)')
legend
legend('NLS','Capon','Location','NorthEast')
xlabel('Frequency [Hz]'), ylabel('Cost function')
grid on;
xlim([fminn,fmaxx])


figure(155)
subplot(2,1,1)
plot(1:SnapShotNumbNLS,NLSFreqVect,1:SnapShotNumbCapon,CaponFreqVect)
grid on;
ylabel('Frequency /zeta [Hz]'), xlabel('Snapshot number')
title('Estimated fundamental freqency')
legend('NLS','Capon','Location','SouthEast')
xlim([1,SnapShotNumbNLS])
%ylim([8,30])

figure(35)
plot(1:SnapShotNumbNLS,TimeNLS,1:SnapShotNumbCapon,TimeCapon)
grid on;
ylabel('Computation time [s]'), xlabel('Snapshot number')
title('Computaiton time')
legend('NLS','Capon','Location','SouthEast')


ww_snap_NLS = fs/2*linspace(0,1,fftlenNLS/2+1);
ww_snap_Capon = fs/2*linspace(0,1,fftlenCapon/2+1);


%% Smooth

yy = smooth(fftt,abs(CostNLS(pss(1),:)),spann,sTYPE).';
% plot raw cost function with LOWESS line
figure(105)
subplot(2,1,1)
plot(fftt, abs(CostNLS(pss(1),:)))
hold on; grid on;
plot(fftt,yy)
title('Cost functions')
xlim([fminn,fmaxx]);
xlabel('Frequency [Hz]'), ylabel('Cost function')
legend('Cost function','LOWESS line','Location','NorthEast')
hold off;

% Normalized, weighted costfunctions for the snapshot of interest
figure(106)
subplot(2,1,1)
plot(fftt, (nwCostNLS(pss,:)))
grid on;
title('Normalized, weighted cost functions')
xlim([fminn,fmaxx]);
xlabel('Frequency [Hz]'), ylabel('Cost function')

figure(106)
subplot(2,1,2)
plot(fftt, (nwCostCapon(pss,:)))
grid on;
xlim([fminn,fmaxx]);
xlabel('Frequency [Hz]'), ylabel('Cost function')

yy = smooth(fftt,abs(CostCapon(pss(1),:)),spann,sTYPE).';

%   Raw cost function Capon with LOWESS line
figure(105)
subplot(2,1,2)
plot(fftt, abs(CostCapon(pss(1),:)))
hold on; grid on;
xlim([fminn,fmaxx]); 
plot(fftt,yy)
xlabel('Frequency [Hz]'), ylabel('Cost function')
legend('Cost function','LOWESS line','Location','NorthEast')
hold off;

%   Plot normalized, weighted NLS Cost function
figure(101);
plot(fftt, (nwCostNLS(pss(1),:)))
title('Normalized, weighted Non-linear Least Squares cost function')
xlabel('Frequency [Hz]'), ylabel('Cost function')
xlim([fminn,fmaxx]);
grid on;

%   Plot normalized, weighted Capon Cost function
figure(102);
plot(fftt, (nwCostCapon(pss(1),:)))
title('Normalized, weighted Capon cost function')
xlabel('Frequency [Hz]'), ylabel('Cost function')
xlim([fminn,fmaxx]);
grid on;

figure(155)
subplot(2,1,2)
plot(1:SnapShotNumbNLS,wNLSFreqVect,1:SnapShotNumbCapon,wCaponFreqVect)
grid on;
ylabel('Frequency [Hz]'), xlabel('Snapshot number')
title('Estimated fundamental frequency, weighted')
legend('NLS','Capon','Location','Northeast')
xlim([1,SnapShotNumbNLS])

figure(fig1 + 5);
plot(fftt,(nwCostNLS(pss,:)),fftt, (nwCostCapon(pss,:)))
title('Cost functions from noise')
legend('NLS','Capon','Location','NorthEast')
xlabel('Frequency [Hz]'), ylabel('Cost function')
grid on;
xlim([fminn,fmaxx])



%% Marked spectrograms

try
    markNLSFreqVect = wNLSFreqVect;
    markCaponFreqVect = wCaponFreqVect;
catch err
    markNLSFreqVect = NLSFreqVect;
    markCaponFreqVect = CaponFreqVect;
end
    
resol = 20;
fonts = 10;

nfft = 2^12;                        % This mighte be low
twindow = hamming(nfft);


figure(42);
set(gcf,'PaperPositionMode','auto')
set(42, 'units', 'centimeters', 'pos', [0 0 18 12])
%spectrogram(signal,twindow,[],[],fs);
[SS,FF,TT] = spectrogram(signal,twindow,[],[],fs);
surf(FF,TT,10*log10(abs(SS)).','EdgeColor','none');
view([0,90])
colormap('autumn');
ylabel('Time [s]'), xlabel('Frequency [Hz]')
set(gca,'FontName','Times New Roman','FontSize',fonts)
set(findall(gcf,'type','text'),'FontName','Times New Roman','FontSize',fonts)
ylim([min(TT),max(TT)])    % bugged in matlab 2015a
xlim([0,fs/2])             % ----------"-----------
hold on


indexingC = 1:resol:length(Tsnap0Capon);
indexingN = 1:resol:length(Tsnap0NLS);

LL_fC = zeros(1,length(indexingC));
LL_fN = zeros(1,length(indexingN));

iind_i = 0;
for iind = indexingC
    iind_i = iind_i + 1;
    ff_f = markCaponFreqVect(iind);
    LL_fC(iind_i) = floor(fs/2/ff_f);
    ff_fv = ff_f*(1:LL_fC(iind_i));
    tt_fv = ones(size(ff_fv))*Tsnap0Capon(iind)/(fs);
    
end

iind_i = 0;
for iind = indexingN
    iind_i = iind_i + 1;
    ff_f = markNLSFreqVect(iind);
    LL_fN(iind_i) = floor(fs/2/ff_f);
    ff_fv = ff_f*(1:LL_fN(iind_i));
    tt_fv = ones(size(ff_fv))*Tsnap0NLS(iind)/(fs);
    
end


%% Capon line

minLC_l = LL_fC == min(LL_fC);
minLCp = find(minLC_l == 1);
minLCp = minLCp(1);
minLC = LL_fC(minLCp);

LLiindex = 1:65;

for LLi = LLiindex
    ffCv = markCaponFreqVect*LLi;
    ttCv = Tsnap0Capon/(fs);
    plot(ffCv,ttCv,'Color','m','LineWidth',0.1)

end


%% NLS line

minLN_l = LL_fN == min(LL_fN);
minLNp = find(minLN_l == 1);
minLNp = minLNp(1);
minLN = LL_fN(minLNp);


for LLi = LLiindex
    ffNv = markNLSFreqVect*LLi;
    ttNv = Tsnap0NLS/(fs);
    plot(ffNv,ttNv,'Color','k','LineWidth',0.1)

end


title('Spectrogram marked with lines indicating estiamted harmonics')
hold off;


figure(201);
set(gcf,'PaperPositionMode','auto')
set(201, 'units', 'centimeters', 'pos', [0 0 18 12])

%spectrogram(signal,twindow,[],[],fs)
[SS,FF,TT] = spectrogram(signal,twindow,[],[],fs);
surf(FF,TT,10*log10(abs(SS)).','EdgeColor','none');
view([0,90])

ylabel('Time [s]'), xlabel('Frequency [Hz]')
colormap('autumn');
set(gca,'FontName','Times New Roman','FontSize',fonts)
set(findall(gcf,'type','text'),'FontName','Times New Roman','FontSize',fonts)
ax = gca;                  % bugged in matlab2015a
ax.XTick = 0:300:1500;     % ---------"-----------
title('Spectrogram input signal')
ylim([min(TT),max(TT)])    % bugged in matlab 2015a
xlim([0,fs/2])             % ----------"-----------

%   Set key figure active
figure(155)

disp('Done.')


%%   Playback sound
%soundsc(signal,fs)
